@@ -2,40 +2,76 @@ __author__ = "hugo.inzirillo"
 
 from abc import ABCMeta, abstractmethod
 
+from napoleontoolbox.authentication import PasswordCredentials, ClientCredentials
 from napoleontoolbox.napoleon_config_tools import getter, config
 
 
 class AbsctractRessource(metaclass=ABCMeta):
 
-    @property
     @abstractmethod
-    def id(self):
+    def get_credentials(self):
         raise NotImplemented
 
     @property
     @abstractmethod
-    def secret(self):
+    def _id(self):
         raise NotImplemented
 
     @property
     @abstractmethod
-    def type(self):
+    def _secret(self):
         raise NotImplemented
 
     @property
     @abstractmethod
-    def user(self):
+    def _type(self):
         raise NotImplemented
 
     @property
     @abstractmethod
-    def password(self):
+    def _user(self):
         raise NotImplemented
 
+    @property
+    @abstractmethod
+    def _password(self):
+        raise NotImplemented
+
+
+class RessourceTemplate(AbsctractRessource):
+
+    def get_credentials(self):
+        pass
+
+    @property
+    def _id(self):
+        pass
+
+    @property
+    @abstractmethod
+    def _secret(self):
+        pass
+
+    @property
+    @abstractmethod
+    def _type(self):
+        pass
+
+    @property
+    @abstractmethod
+    def _user(self):
+        pass
+
+    @property
+    @abstractmethod
+    def _password(self):
+        pass
+
 
 @config
 class Service(AbsctractRessource):
     def __init__(self):
+        self.__credentials = PasswordCredentials()
         self.__id = None
         self.__secret = None
         self.__type = None
@@ -43,85 +79,97 @@ class Service(AbsctractRessource):
         self.__password = None
 
     @property
-    def id(self):
+    def _id(self):
         return self.id
 
     @property
-    def secret(self):
-        return self.secret
+    def _secret(self):
+        return self.__secret
 
     @property
-    def type(self):
-        return self.type
+    def _type(self):
+        return self.__type
 
     @property
-    def user(self):
-        return self.user
+    def _user(self):
+        return self.__user
 
     @property
-    def password(self):
-        return self.password
+    def _password(self):
+        return self.__password
+
+    def get_credentials(self):
+        return PasswordCredentials(self._id, self._secret, self._user, self._password)
 
 
 @config
 class Provider(AbsctractRessource):
     def __init__(self):
+        self.__credentials = None
         self.__id = None
         self.__secret = None
         self.__type = None
 
     @property
-    def type(self):
-        return self.type
+    def _type(self):
+        return self.__type
 
     @property
-    def id(self):
-        return self.id
+    def _id(self):
+        return self.__id
 
     @property
-    def secret(self):
-        return self.secret
+    def _secret(self):
+        return self.__secret
 
-    def user(self):
-        pass
+    @property
+    def _user(self):
+        return None
 
-    def password(self):
-        pass
+    @property
+    def _password(self):
+        return None
+
+    def get_credentials(self):
+        return ClientCredentials(self._id, self._type)
 
 
 class Bitmex(Provider):
 
     @property
     @getter("providers.bitmex.id")
-    def id(self):
-        return self.id
+    def _id(self):
+        return self._id
 
     @property
     @getter("providers.bitmex.secret")
-    def secret(self):
-        return self.secret
+    def _secret(self):
+        return self._secret
 
     @property
     @getter("providers.bitmex.type")
-    def type(self):
-        return self.type
+    def _type(self):
+        return self._type
+
+    def get_credentials(self):
+        return super(Bitmex, self).get_credentials()
 
 
 class Dropbox(Provider):
 
     @property
     @getter("providers.dropbox.id")
-    def id(self):
+    def _id(self):
         return self.id
 
     @property
     @getter("providers.dropbox.secret")
-    def secret(self):
+    def _secret(self):
         return self.secret
 
     @property
     @getter("providers.dropbox.type")
-    def type(self):
+    def _type(self):
         return self.type
 
 
@@ -129,35 +177,35 @@ class Binance(Provider):
 
     @property
     @getter("providers.binance.id")
-    def id(self):
-        return self.id
+    def _id(self):
+        return self._id
 
     @property
     @getter("providers.binance.secret")
-    def secret(self):
-        return self.secret
+    def _secret(self):
+        return self._secret
 
     @property
     @getter("providers.binance.type")
-    def type(self):
-        return self.type
+    def _type(self):
+        return self._type
 
 
 class CryptoCompare(Provider):
 
     @property
     @getter("providers.cryptocompare.id")
-    def id(self):
+    def _id(self):
         return self.id
 
     @property
     @getter("providers.cryptocompare.secret")
-    def secret(self):
+    def _secret(self):
         return self.secret
 
     @property
     @getter("providers.cryptocompare.type")
-    def type(self):
+    def _type(self):
         return self.type
 
 
@@ -165,29 +213,31 @@ class NapoleonService(Service):
 
     @property
     @getter("providers.napoleon_service.id")
-    def id(self):
+    def _id(self):
         return self.id
 
     @property
     @getter("providers.napoleon_service.secret")
-    def secret(self):
+    def _secret(self):
         return self.secret
 
     @property
     @getter("providers.napoleon_service.type")
-    def type(self):
+    def _type(self):
         return self.type
 
     @property
     @getter("providers.napoleon_service.user")
-    def user(self):
+    def _user(self):
         return self.user
 
     @property
     @getter("providers.napoleon_service.password")
-    def password(self):
+    def _password(self):
         return self.password
-
+    @property
+    def credentials(self):
+        return super(NapoleonService, self).get_credentials()
 
 class Ressources(object):
     def __init__(self):
@@ -216,3 +266,9 @@ class Ressources(object):
     @property
     def napoleon_service(self):
         return self.__napoleon_service
+
+
+if __name__ == '__main__':
+    s = Bitmex().get_credentials()
+    credentials = NapoleonService().get_credentials()
+    end = True
