@@ -96,11 +96,15 @@ class CryptoCompare:
         return output
 
 
-def binance_get_histo_balance(api, p, request_date):
+def binance_get_histo_future_balance(api, p, request_date):
     bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
     return bot.get_balance_snapshot(request_date)
 
+def binance_get_futures_transfer_histo(api, p, start_time, end_time):
+    bot = binance_bot.NaPoleonBinanceFutureBot(api, p)
+    return bot.get_futures_transfer_history(['USDT', 'BNB'], start_time, end_time)
 
+#https://binance-docs.github.io/apidocs/spot/en/#wallet-endpoints
 def binance_get(api, p, type_, start_time, end_time = None, symbol = None, aggreg = None):
   '''
   Accepted types: futures_funding, all_futures_orders, futures_klines (aggreg = 1H), spot_klines (aggreg = '1m', '1h')
@@ -407,8 +411,12 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
     ohlc_btc_cc_pkl_file_name = f'{filename_stub}_ohlc_btc_cc.pkl'
     ohlc_eth_cc_pkl_file_name = f'{filename_stub}_ohlc_eth_cc.pkl'
     balance_pkl_file_name = f'{filename_stub}_balance.pkl'
+    transfer_pkl_file_name = f'{filename_stub}_transfer.pkl'
 
-    initial_balances = binance_get_histo_balance(public_key, private_key, start_time)
+    fut_transfers = binance_get_futures_transfer_histo(public_key, private_key, start_time, end_time)
+
+
+    initial_balances = binance_get_histo_future_balance(public_key, private_key, start_time)
 
     trades = binance_get(public_key, private_key, 'trade_history', start_time, end_time=end_time, symbol ='ETHUSDT').drop_duplicates()
     ohlc_eth_binance = binance_get(public_key, private_key, 'futures_klines', start_time, end_time= end_time, symbol ='ETHUSDT', aggreg ='1m')
@@ -584,6 +592,32 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
     wh = wh.groupby('dt_floor_h').sum()
     wh['pnl'] = wh['realized_pnl']
 
+    if fut_transfers is None:
+        wh['transfer_usd_bis'] = 0.
+    else:
+        wh['transfer_usd_bis'] = 0.
+
+    fut_transfers_bnb = fut_transfers[fut_transfers['asset'] == 'BNB'].copy()
+    fut_transfers_usd = fut_transfers[fut_transfers['asset'] == 'USDT'].copy()
+
+    fut_transfers_bnb = fut_transfers_bnb.set_index('date')
+    fut_transfers_usd = fut_transfers_usd.set_index('date')
+
+    fut_transfers_usd['dt_floor_h'] = fut_transfers_usd.index.map(lambda x: x.floor('H'))
+    fut_transfers_usd = fut_transfers_usd.groupby('dt_floor_h').sum()
+
+    fut_transfers_bnb['dt_floor_h'] = fut_transfers_bnb.index.map(lambda x: x.floor('H'))
+    fut_transfers_bnb = fut_transfers_bnb.groupby('dt_floor_h').sum()
+
+    fut_transfers_usd = fut_transfers_usd[['amount']].copy()
+    fut_transfers_usd.columns = ['amount_futures_transfer_usd']
+
+    fut_transfers_bnb = fut_transfers_bnb[['amount']].copy()
+    fut_transfers_bnb.columns = ['amount_futures_transfer_bnb']
+
+    wh = pd.merge(wh, fut_transfers_usd, how='left', left_index=True, right_index=True)
+    wh = pd.merge(wh, fut_transfers_bnb, how='left', left_index=True, right_index=True)
+
     initial_bnb_balance = 0
     initial_usdt_balance = 0
     try:
